/*
*    Language Investigator and Stractural Analyzer (LISA) -
*    A descriptive programming language
*  LICENCE:        LGPL
 * FILE:			symboltable.h
 *
 * PURPOSE:		Symbol table definitions and declarations.
 *
 * CREATED AT:		November 6, 2011
 *
 * AUTHOR: 		Seyed Pooria Madani Kochak (s.p.madani@gmail.com)
 *
 * Project Manager:	Dr. Cezar Campeanu (ccampeanu@upei.ca)
 */

/*
 * This file holds the Symbol Table's definitions and definitions of functions
 * that manipulates symbol table.
 *
 * Revision History
 * --------------------------------------------------------------------------
 * Seyed Pooria Madani - Creation							November 6, 2011
 *
 * Brandon Stewart     - JSON symbol loading, namespace support, and restructuring, details in
 *                     "lisa-changelist.txt".                      June/July 2015
*
 */

#ifndef SYMBOLTABLE_H_INCLUDED
#define SYMBOLTABLE_H_INCLUDED

#include "stack.h"

#include "nxjson/nxjson.h"

Stack* namespaceStack;// Holds the namespaces that follow the "using ..." statement
					  //The namespace at the top of the stack takes priority


enum symbol_node_type {
	NODETYPE_NULL, NODETYPE_DFA, NODETYPE_NFA, NODETYPE_REGEX, NODETYPE_STRING, //String literals
	NODETYPE_INTEGER,
	NODETYPE_BOOLEAN, //true false
	NODETYPE_FMPOINTER
//This is an inner node for compiler managin the heap
};

/*
 * Types that the lexical scanner will detect and record and return back to the parser.
 */
enum symbol_token_type {
	TOKENTYPE_ID,
	TOKENTYPE_STRINGLITERALS,
	TOKENTYPE_NUMBER,
	TOKENTYPE_KEYWORD,
	TOKENTYPE_PREDEFINED_FUNCTION,
	TOKENTYPE_SYMBOL,
	TOKENTYPE_NAMESPACE
};

/*
 * This gives the intermediate code generator the data necessary to check for errors
 */
struct function_data {
	unsigned int num_params;
	enum symbol_node_type* param_types;
	int returns_pointer;

};

typedef struct {
	void *value_name; //this is the string name that the value get referenced by
	void *value; //This is the value held by the application
	int namespace_type;
	enum symbol_token_type token_type;
	enum symbol_node_type node_type;
	int array_capacity;

	struct function_data param_data;
} symbol_node;

/*************************************************************/
//		Symbol Table constants
/*************************************************************/
#define INIT_CAPACITY 	100 //initial capacity of symbol table
#define INCREASE_BY 	100	//Every time that we increase the table - we increase it by this value
/*************************************************************/

/*************************************************************/
//		Symbol Table instance variables
/*************************************************************/
extern symbol_node *symbol_table;
extern int number_of_nodes;
extern int capacity_of_symbol_table;
/*************************************************************/

/*************************************************************/
//		Symbol Table functions
/*************************************************************/

/*
 * This function initilize the symbol table with all the
 * predefined keywords, functions, and operators.
 */
void init_symbol_table(void);

void load_json_with_namespace(char*);

void load_symbols_from_json(int, int, const nx_json*);

//frees the memory used by the symbol_table
void delete_symbol_table(void);

/*
 * This function will expand the size of the symbol table
 * by a constant (INCREASE_BY) every time symbol table reaches
 * its capacity
 */
void expand_symbol_table(void);

/*
 * Add a general symbol node into the symbol table.
 * It will check for uniqueness of the symbol, if is already on the symbol table
 * it will return index to that item.
 */
int add_symbol_node(symbol_node *);

/*
 * Will find a symbol in symbol table based on value_name.
 */
int find_symbol_node(symbol_node *);

/*
 * This function will print the symbol table for debugging purposes.
 */
void print_symbol_table(void);

/*
 * This function will add Identifier node into symbol table.
 */
int add_id_symbol_node(char *);

/*
 * This function will add INTEGER node into symbol table.
 */
int add_number_symbol_node(char *, int);

/*
 * This function will add STRING literals into symbol table.
 */
int add_stringliterals_symbol_node(char *);

int add_character_symbol_node(char *);

int add_finitemachine_pointer_symbol_node(char *);

/*
 * The node_type of a symbol table item will get set.
 * This method will get called once parser finds out
 * about type of a variable in the declaration part.
 */
void set_variable_type(int, int);
/*************************************************************/
//same as above, but with namespace id
void namespace_set_variable_type(int, int, int);

/*
 * The node_type of a symbol table item will get set.
 * This method will get called once parser finds out
 * about type of a variable in the declaration part.
 */
void set_array_type(int, int, int);
/*************************************************************/
//same as above, but with namespace id
void namespace_set_array_type(int, int, int, int);


//counts symbols in other namespaces that have the same details
int count_symbols_same_declaration(symbol_node *);

#endif
